#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <err.h>

#include "chip.h"

#define MIN_ALINES	13
#define MAX_ALINES	19

/*
   A13 - 16k	// npr. 27C128
   A14 - 32k	// npr. 27C256
   A15 - 64k	// npr. 27C512
   A16 - 128k	// npr. 27C010, 27C1001
   A17 - 256k	// npr. 27C020, 27C2001
   A18 - 512k	// npr. 27C040, 27C4001
 */

/*
   Jumper0 - ramcs2 -> a17
   Jumper1 - ramcs2 -> 1
   Jumper2 - - (vcc -> pin32)
   Jumper3 - vcc -> pin28 (malo kučište)
 */

Chip::Chip(const char *name) {
	if (!name || !ConfigureFrom(name)) {
		a = MAX_ALINES;
		ConfigureJumpers();
	}
}

bool Chip::ConfigureFrom(const char *name) {
	// zadan broj adresnih linija?
	if (tolower(name[0]) == 'a') {
		if (sscanf(name, "a%d", &a) != 1) {
			warnx("Invalid chip type");
			return false;
		}
		if (a < MIN_ALINES || a > MAX_ALINES) {
			warnx("Invalid number of address lines");
			return false;
		}
		ConfigureJumpers();
		return true;
	}

	// naziv chipa
	if (!strncasecmp(name, "27c", 3))
		name += 3;

	static const struct _chips {
		const char *name;
		int a;
	} chips[] = {
		{"128", 13},
		{"256", 14},
		{"512", 15},
		{"010", 16}, {"1001", 16},
		{"020", 17}, {"2001", 17},
		{"040", 18}, {"4001", 18},
		{NULL, 0}
	};

	for (const _chips *c = chips; c->name; ++c) {
		if (!strcmp(name, c->name)) {
			a = c->a;
			ConfigureJumpers();
			return true;
		}
	}

	warnx("Unknown chip type");
	return false;
}

bool Chip::Is28Pin() const {
	return a >= 16;
}

bool Chip::HasA(int i) const {
	return a >= i;
}

const bool * Chip::GetJumperSettings() const {
	return jumpers;
}

unsigned long Chip::GetTROMAddress() const {
	unsigned long tra = 0;
	for (int i = MIN_ALINES; i <= MAX_ALINES; ++i)
		tra |= !HasA(i) << i;
	return tra;
}

unsigned long Chip::Size() const {
	return (1 << (a+1)) - 1;
}

void Chip::ConfigureJumpers() {
	jumpers[0] = true;	// CS2 = a17
	jumpers[1] = false;
	jumpers[2] = Is28Pin();
	jumpers[3] = !Is28Pin();
}
