#include <stdio.h>
#include <ftdi.h>

#include "ftdidrv.h"

FtdiDrv::FtdiDrv() {
	ftdi = ftdi_new();
	if (!ftdi) {
		fprintf(stderr, "Can't init ftdi!\n");
		exit(1);
	}

	pos = 0;
}

FtdiDrv::~FtdiDrv() {
	if (ftdi) {
		Flush();
		ftdi_free(ftdi);
	}
}

bool FtdiDrv::Init() {
	if (ftdi_usb_open(ftdi, 0x0403, 0x6001) < 0) {
		fprintf(stderr, "Ne mogu otvoriti ftdi device!\n");
		FtdiError();
		return false;
	}

	if (ftdi_enable_bitbang(ftdi, 0xFD) < 0) {
		fprintf(stderr, "Ne mogu prebaciti u bitbang mode!\n");
		FtdiError();
		return false;
	}

	if (ftdi_set_baudrate(ftdi, 300000) < 0) {
		fprintf(stderr, "Ne mogu postaviti baudrate. Preskačem...\n");
		FtdiError();
	}

	return TROM::Init();
}

void FtdiDrv::Write(bool wr) {
	SetBit(0, wr);
//	if (wr) Flush();	// fore radi... :)
}

void FtdiDrv::Inc(bool i) {
	SetBit(3, i);
}

void FtdiDrv::SCK(bool c) {
	SetBit(2, c);
}

void FtdiDrv::SDA(bool d) {
	SetBit(4, d);
}

bool FtdiDrv::SFB() {
	unsigned char c;

	Flush();
	if (ftdi_read_pins(ftdi, &c) < 0)
		FtdiFatal();

	return !!(c & 2);
}

void FtdiDrv::SetBit(int i, bool b) {
	last = b ? (last | ((unsigned char)1 << i))
		 : (last & ~((unsigned char)1 << i));
	SendBits(last);
}

void FtdiDrv::SendBits(unsigned char b) {
	if (pos >= (int)sizeof(buffer))
		Flush();
	buffer[pos++] = b;
}

void FtdiDrv::Flush() {
	if (ftdi_write_data(ftdi, buffer, pos) < 0)
		FtdiFatal();
	pos = 0;
}

void FtdiDrv::FtdiError() {
	fprintf(stderr, "Ftdi error: %s\n", ftdi_get_error_string(ftdi));
}

void FtdiDrv::FtdiFatal() {
	FtdiError();
	ftdi_free(ftdi);
	ftdi = NULL;
	pos = 0;
	exit(1);
}
