#include "trom.h"

#define SCKCK() {SCK(false); SCK(true);}
#define INC() {Inc(false); Inc(true);}

void TROM::Shutdown() {
	// shutdown
	SDA(false);
	Write(true);
	SCK(false);
	Inc(false);
}

bool TROM::Init() {
	WriteMode(false);
	return true;
}

void TROM::WriteMode(bool wr) {
	if (wr_mode == wr)
		return;
	if (wr) {
		cur_ha = 0;
		Inc(true);
		SCK(true);
	}
	Write(!wr);
	wr_mode = wr;
}

void TROM::SetHighAddr(byte_t ah, bool ce2) {
	// ah2 ce2 ah1 ah0
	SetHighAddr((ah & 3) | (ce2 << 2) | ((ah & 4) << 1));
}

// mora biti prije setdata
// poretpostavka je da je omogućen ha reg
void TROM::SetHighAddr(byte_t ha) {
	int tikova;
	tikova = (cur_ha > ha) ? 0x100 - cur_ha + ha : ha - cur_ha;
	while (tikova--) SCKCK();
	cur_ha = ha;
}

// pretpostavka je da je onemogućen ha reg
void TROM::SetData(byte_t d) {
	// pošalje podatak
	for (int i = 0; i < 8; ++i, d>>=1) {
		SDA(d & 1);
		SCKCK();
	}
}

TROM::byte_t TROM::GetData() {
	byte_t d = 0;
	for (int i = 0; i < 8; ++i) {
		d = (d >> 1) | (SFB() << 7);
		SCKCK();
	}
	return d;
}

bool TROM::Test() {
	bool ok = true;

	WriteMode(true);

	// dummy
	SetData(0);
	GetData();

	// 0x55 test
	SetData(0x55);
	if (GetData() != 0x55) ok = false;

	// 0xAA test
	SetData(0xAA);
	if (GetData() != 0xAA) ok = false;

	SCKCK();	// "flush", makar nepotreban

	WriteMode(false);
	return ok;
}

void TROM::SetAddress15(unsigned short int addr, bool ce) {
	SetAddress((addr << 1) | (!ce));
}

void TROM::SetAddress(unsigned short int addr) {
	while (addr--) INC();
	INC(); // dodatni RCK
}

void TROM::SendData(const void *d, unsigned long size, unsigned long addr, sendcallback sb, void *arg) {
	const byte_t *dat = (const byte_t*)d;
	byte_t ah, odata;
	unsigned long s = 1;

	if (!size)
		return;

	WriteMode(true);	// /CE = ? !

	ah = addr >> 15;
	SetHighAddr(ah);	// CS2 = A17

	{
	Inc(false);	// /CE = 0, addr = 0, onemogući ha reg.
	
	// pripremi prvi podatak
	SetData(*dat);
	SCKCK();	// "flush"
	odata = *(dat++);
	Inc(true);
	}

	// došeće se do početne lokacije (ali bez visokih bitova)
	// (uništava podatke!)
	for (unsigned long a = 0; a < (addr & 0x7FFF)*2; ++a)
		INC();

	bool cfginc = false;
	for (; s <= size; ++s, ++dat) {
		{
		Inc(false);		// /CE = 1 (zapiše!)

		if (cfginc) {
			SCKCK();	// "flush" ha reg
			if (s < size)
				odata = ~*dat;	// uništen je podatak
		}

		Inc(true);
		}

		if (sb) sb(s, size, arg);	// pozove callback

		// ako ima još
		if (s < size) {
			// uveća adresu
			ah = ++addr >> 15;

			// da li treba uvećati visoke adrese prije sljedećeg ciklusa?
			cfginc = ah != ((addr+1) >> 15);


			{
			Inc(false);	// /CE = 0, addr++

			// postavi novi podatak ako se razlikuje od prethodnog
			// ili ako treba uvećati adresu, pa bi podatak bio uništen
			if (odata != *dat || cfginc) {
				SetData(*dat);
				if (!cfginc) SCKCK();	// "flush" data
				odata = *dat;
			}

			Inc(true);
			}

			if (cfginc) {
				SCKCK();	// "flush" data, ha++
			}
		}
	}

	WriteMode(false);
}
